@@ -5,8 +5,8 @@
 # No counter-examples are known to this test.
 
 # Algorithm: given an odd integer n, that is not a perfect power:
-#   1. Perform a base-2 Fermat test.
-#   2. Find the first P>0 such that kronecker(n, P^2 + 4) = -1.
+#   1. Perform a (strong) base-2 Fermat test.
+#   2. Find the first P>0 such that kronecker(P^2 + 4, n) = -1.
 #   3. If the Lucas U sequence: U(P, -1, n+1) = 0 (mod n), then n is probably prime.
 
 # See also:
@@ -21,26 +21,32 @@ use experimental qw(signatures);
 use lib qw(../lib);
 use Math::AnyNum qw(is_prime is_power lucasUmod kronecker powmod);
 
+sub findP ($n) {
+
+    # Find P such that kronecker(P^2 + 4, n) = -1.
+    for (my $k = 1 ; ; ++$k) {
+        if (kronecker($k * $k + 4, $n) == -1) {
+            return $k;
+        }
+    }
+}
+
 sub PSW_primality_test ($n) {
 
     return 0 if $n <= 1;
     return 1 if $n == 2;
+
+    return 0 if !($n & 1);
     return 0 if is_power($n);
 
     # Fermat base-2 test
     powmod(2, $n - 1, $n) == 1 or return 0;
 
-    # Find P such that kronecker(n, P^2 + 4) = -1.
-    my $P;
-    for (my $k = 1 ; ; ++$k) {
-        if (kronecker($n, $k * $k + 4) == -1) {
-            $P = $k;
-            last;
-        }
-    }
+    my $P = findP($n);
+    my $Q = -1;
 
     # If LucasU(P, -1, n+1) = 0 (mod n), then n is probably prime.
-    lucasUmod($P, -1, $n + 1, $n) == 0;
+    lucasUmod($P, $Q, $n + 1, $n) == 0;
 }
 
 #
